/*
  libreiserfs - a library for manipulating reiserfs partitions
  Copyright (C) 2001-2004 Yury Umanets <torque@ukrpost.net>.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 2 of the License, or (at your option) any later
  version.
                                                                                                 
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.
                                                                                                 
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#include <dal/file_dal.h>
#include <reiserfs/reiserfs.h>
#include <progs/tools.h>

int main(int argc, char *argv[]) {
	dal_t *dal;
	reiserfs_fs_t *fs;
	reiserfs_dir_t *dir;
	reiserfs_path_node_t *leaf;
	reiserfs_dir_entry_t entry;
	long offset = 0;

	if (argc < 3) {
		fprintf(stderr, "Usage: %s DEV DIR [ FROM ]\n", argv[0]);
		return 0xff;
	}

	if (!(dal = file_dal_open(argv[1], DEFAULT_BLOCK_SIZE, O_RDONLY))) {
		libreiserfs_exception_throw(EXCEPTION_ERROR, EXCEPTION_CANCEL, 
					    "Couldn't create device abstraction "
					    "for %s.", argv[1]);
		return 0xfe;    
	}
    
	if (!(fs = reiserfs_fs_open_fast(dal, dal))) {
		libreiserfs_exception_throw(EXCEPTION_ERROR, EXCEPTION_CANCEL, 
					    "Couldn't open filesystem on %s.",
					    argv[1]);
		goto error_free_dal;
	}
    
	if (!(dir = reiserfs_dir_open(fs, argv[2]))) {
		libreiserfs_exception_throw(EXCEPTION_ERROR, EXCEPTION_CANCEL, 
					    "Couldn't open dir %s.", argv[2]);
		goto error_free_fs;
	}	
    
	if (argc > 3 && (offset = progs_str2long(argv[3], -1)) == -1) {
		libreiserfs_exception_throw(EXCEPTION_ERROR, EXCEPTION_CANCEL, 
					    "Invalid offset %s.", argv[3]);
		goto error_free_fs;
	}
    
	if (!reiserfs_dir_seek(dir, (uint32_t)offset)) {
		libreiserfs_exception_throw(EXCEPTION_ERROR, EXCEPTION_CANCEL, 
					    "Couldn't seek to %lu. Current "
					    "position is %lu.", offset, 
					    reiserfs_dir_offset(dir));
		goto error_free_dir;
	}
	
	while (reiserfs_dir_read(dir, &entry))
		printf("%s\n", entry.de_name);
    
	reiserfs_dir_close(dir);
    
	reiserfs_fs_close(fs);
	file_dal_close(dal);
    
	return 0;

error_free_dir:
	reiserfs_dir_close(dir);
error_free_fs:
	reiserfs_fs_close(fs);
error_free_dal:
	file_dal_close(dal);
error:
	return 0xff;    
}

